<?php
/**
 * Plugin Name: PlantUML plugin for WordPress
 * Plugin URI: https://bitbucket.org/litcoder/plantuml-plugin-for-wordpress
 * Description: This plugin allows you to draw UMLs with PlantUML grammar in WordPress. Describe your UML surrounded by div tags with plantuml class like "&lt;div class="plantuml"&gt;Bob -> Alice&lt;/div&gt;"
 * Version: 0.1
 * Author: Litcoder
 * Auther URI: http://www.litcoder.com
 * License: GPL2 or later
 */

require_once("WpuDefs.php");
require_once("WpuDebug.php");
require_once("WpuUMLWebCreator.php");
require_once("WpuUMLLocalCreator.php");


function wpu_to_image($matches)
{
    // Local UML creator
    if (WPU_LOCAL_CREATOR) {
        $creator = new WpuUMLLocalCreator;
        $outFormat = '%s';
    } 
    // Web UML creator
    else {
        $creator = new WpuUMLWebCreator;
        $outFormat = '<img src=%s>';
    }

    $img = $creator->create($matches[0]);
    $imgTag = sprintf($outFormat, $creator->create($matches[0]));

    return $imgTag;
}


function wpu_get_tag_regex()
{
    $tag = "div";
    $class = "plantuml";

    $regex = 
        "#(?<!\$)<\s*" 
        . $tag
        . "\s+class\s*=\s*[\'\"]" 
        . $class
        . "[\'\"]\s*>(.*?)<\s*\/"
        . $tag
        . "\s*>#msi";

    WpuDebug::info("UML fetching regex", htmlspecialchars($regex));

    return $regex;
}


function wpu_draw_uml_in($content)
{
    $regex = wpu_get_tag_regex();

    $content = preg_replace_callback(
        $regex,
        'wpu_to_image', $content);

    return $content;
}


/*
  Actions and filters.
*/

add_filter('the_content', 'wpu_draw_uml_in');

?>
